//
//  ContainerMainView.swift
//  SideBarMenuSwift3


import UIKit


enum SideMenuState{
    case Closed
    case Opened
}

@objc
protocol SideMenuDelegate {
    @objc optional func toogle_side_menu()
    @objc optional func side_menu_enabled()
    @objc optional func side_menu_disabled()
    
}

var sidemenudelegate:SideMenuDelegate? = nil

class ContainerMainView: UIViewController,SideMenuDelegate {

    var centerViewController: MainView!
    var centerNavigationController:UINavigationController!
    var sidemenu_width: CGFloat = 220
    var side_menu_controller: SideMenuController?
    var can_i_open:Bool = true
    
    var side_menu_state: SideMenuState = .Closed
        {
        didSet{
            let ShowShadow = side_menu_state != .Closed
            showShadowForCenterViewController(shouldShowShadow: ShowShadow)
        }
    }
    

    
    override func viewDidLoad() {
        super.viewDidLoad()

        
        let main_storyboard = UIStoryboard(name: "Main", bundle: Bundle.main)
        
        centerViewController = main_storyboard.instantiateViewController(withIdentifier: "MainView") as? MainView
        centerViewController.ContainerMainView = self
        
        centerNavigationController = UINavigationController(rootViewController: centerViewController)
        view.addSubview(centerNavigationController.view)
        addChildViewController(centerNavigationController)
        
        centerNavigationController.didMove(toParentViewController: self)
        
        
        let panGestureRecognizer = UIPanGestureRecognizer(target: self, action: #selector(ContainerMainView.handlePanGesture(_:)))
        
        centerNavigationController.view.addGestureRecognizer(panGestureRecognizer)
        
        sidemenudelegate = self
    }

    
    
    
    func toogle_side_menu()
    {
        togglePanel()
    }
    
    
    
    
    func togglePanel() {
        
        if(can_i_open == false)
        {
            return
        }
        
        let notAlreadyExpanded = (side_menu_state != .Opened)
        if notAlreadyExpanded{
            addPanelViewController()
        }
        animatePanel(shouldExpand: notAlreadyExpanded)
    }
    
    func collapseSidePanels() {
        switch (side_menu_state){
        case .Opened:
            togglePanel()
        default:
            break
        }
    }
    
    func addPanelViewController() {
        if(side_menu_controller == nil){
            let main_storyboard = UIStoryboard(name: "Main", bundle: Bundle.main)
            
            side_menu_controller = main_storyboard.instantiateViewController(withIdentifier: "SideMenuController") as? SideMenuController
            
            side_menu_controller?.ContainerMainView = self
            
            
            addChildSidePanelController(menu: side_menu_controller!)
        }
    }
    
    
    func addChildSidePanelController(menu: SideMenuController) {
        view.insertSubview(menu.view, at: 0)
        addChildViewController(menu)
        menu.didMove(toParentViewController: self)
    }
    
    
    func animatePanel(shouldExpand: Bool) {
        if (shouldExpand) {
            side_menu_state = .Opened
            animateCenterPanelXPosition(targetPosition: sidemenu_width)
            
        } else {
            animateCenterPanelXPosition(targetPosition: 0) { _ in
                self.side_menu_state = .Closed
                                
                self.side_menu_controller!.view.removeFromSuperview()
                self.side_menu_controller = nil;
            }
        }
    }
    
    func animateCenterPanelXPosition(targetPosition: CGFloat, completion: ((Bool) -> Void)! = nil) {
        UIView.animate(withDuration: 0.5, delay: 0, usingSpringWithDamping: 0.8, initialSpringVelocity: 0, options: .curveEaseInOut, animations: {
            self.centerNavigationController.view.frame.origin.x = targetPosition
        }, completion: completion)
    }
    
    func showShadowForCenterViewController(shouldShowShadow: Bool) {
        if (shouldShowShadow) {
            centerNavigationController.view.layer.shadowOpacity = 0.6
            centerNavigationController.view.layer.shadowRadius = 15
        } else {
            centerNavigationController.view.layer.shadowOpacity = 0.0
        }
    }
    
    
    
    
    func handlePanGesture(_ recognizer: UIPanGestureRecognizer) {
        
        if(can_i_open == false)
        {
            return
        }
        
        
        let gestureIsDraggingFromRightToLeft = (recognizer.velocity(in: view).x < 0)
        
        switch(recognizer.state){
        case .began:
            if(side_menu_state == .Closed){
                if(gestureIsDraggingFromRightToLeft == false){
                    addPanelViewController()
                }
                showShadowForCenterViewController(shouldShowShadow: true)
            }
            
        case .changed:
            let screen_center = recognizer.view!.frame.width/2
            let new_center = recognizer.view!.center.x+recognizer.translation(in: view).x
            if(screen_center <= new_center)
            {
                recognizer.view!.center.x = new_center
                recognizer.setTranslation(CGPoint.zero, in: view)
            }
            
            
        case .ended:
            if(side_menu_controller != nil)
            {
                let rec_center = recognizer.view!.center.x
                let screen_center = recognizer.view!.frame.width/2
                if(abs(screen_center-rec_center) > 20)
                {
                    
                    let direction = ( (recognizer.velocity(in: view).x > 10))
                    animatePanel(shouldExpand: direction)
                    
                }
                else
                {
                    let open = abs(screen_center - rec_center) > 40
                    animatePanel(shouldExpand: open)
                }
                
            }
            
        default:
            break
        }
    }
    
    
    func side_menu_enabled()
    {
        can_i_open = true
    }
    
    func side_menu_disabled()
    {
        can_i_open = false
    }
    
    
    
    
    
    

    

}
